import pandas as pd
import os
import numpy as np
import scipy.stats as stats
import matplotlib.pyplot as plt
from statsmodels.stats.multicomp import pairwise_tukeyhsd
import statsmodels.stats.weightstats as ssw
import seaborn as sns
from itertools import combinations
import pingouin as pg
import glob
def Remove_Outlier_Indices(data):
    # find Q1, Q3, and interquartile range for each column
    Q1 = data.quantile(q=.25)
    Q3 = data.quantile(q=.75)
    IQR = data.apply(stats.iqr)
    low = Q1 - 1.5 * IQR
    up = Q3 + 1.5 * IQR
    data_clean = data < low
    # only keep rows in dataframe that have values within 1.5*IQR of Q1 and Q3
    data_clean = data[~((data < low) | (data > up)).any(axis=1)]
    #data_clean = data[~((data < (Q1 - 1.5 * IQR)) | (data > (Q3 + 1.5 * IQR))).any(axis=1)]
    return data_clean
def Limits_Outliers(data):
    Q1 = data.quantile(q=.25)
    Q3 = data.quantile(q=.75)
    IQR = data.apply(stats.iqr)
    low = Q1 - 1.5 * IQR
    up = Q3 + 1.5 * IQR
    return low, up
def Remove_Outlier_Indices(df):
    Q1 = df.quantile(0.25)
    Q3 = df.quantile(0.75)
    IQR = Q3 - Q1
    trueList = ~((df < (Q1 - 1.5 * IQR)) |(df > (Q3 + 1.5 * IQR)))
    return trueList


sns.set_theme(style="ticks", color_codes=True)
C2M_df = pd.DataFrame()
xyz_path = r"C:\Users\Thomas\Documents\xyz"
model_path_in = r"D:\TFE_thomas_final\final_church"
project_path = r"D:\TFE_thomas_final\Final_result_noImages"

os.chdir(model_path_in)
models = glob.glob('*.obj')
#-------------------------------------------------------------------------------

for model in models:
    model_name = model.split('.')
    print(model_name[0], '---------------------------')
    model_path = os.path.join(project_path, model_name[0])

    tukeys_temp2 = []
    model_name_list = []
    for root, dirs, files in os.walk(model_path):
        for name in files:
            if name.startswith("C2M_dc_") and name.endswith(".xyz") and not name.endswith("_maxDist10cm.xyz"):
                xyz = os.path.join(root, name)
                print(xyz)
                rec_name = name.split('.')
                print(rec_name[0])
                model_name_list.append(rec_name[0])

                df = pd.read_csv(xyz, header=0, index_col=0, sep=',')
                df = df.reset_index()
                dist_C2M = df['C2M signed distances']
                # random sample data
                dist_C2M = dist_C2M.sample(n=200000, ignore_index=True)

                # fig, axes = plt.subplots(2, 1)
                # sns.histplot(data=dist_C2M, kde=True, bins=50, stat='count', label='With outliers', ax=axes[0]).set(title=str(rec_name[0])+' reconstruction')
                dist_C2M_clean = Remove_Outlier_Indices(dist_C2M)
                limits = Limits_Outliers(dist_C2M)
                # sns.histplot(data=dist_C2M_clean, kde=True, bins=50, stat='count', label='With outliers', ax=axes[1])
                # Create histogram
                plt.subplot(2, 1, 1)
                plt.hist(dist_C2M, bins=100, label='With outliers')
                plt.legend(loc="upper left")
                plt.ylabel('point count')

                plt.subplot(2, 1, 2)
                print(str(len(dist_C2M_clean)) + ' points left from ' + str(len(dist_C2M)) + ' points original dataset')
                outlier_percent = 100 - ((len(dist_C2M_clean)/len(dist_C2M))*100)
                print('ouliers represent '+str(round(outlier_percent,2))+' % of dataset')
                plt.hist(dist_C2M_clean, bins=100, label='Without outliers')
                plt.legend(loc="upper left")
                plt.ylabel('point count')
                plt.xlabel('dist [m]')
                plt.suptitle('Histogram of dist btw ' + str(rec_name[0]) + ' model and ref for '+str(model_name[0]))
                plt.show()

                # Save to C2M_df
                C2M_df[str(rec_name[0])] = dist_C2M

                tukeys_temp1 = dist_C2M
                tukeys_temp1['name'] = rec_name[0]
                tukeys_temp2.append(tukeys_temp1)

    tukeys = pd.concat(tukeys_temp2, ignore_index=True)
    data = C2M_df
    data_clean = Remove_Outlier_Indices(data)
    col_name_list = []
    for col_name in data.columns:
        col_name_list.append(col_name)
    plt.subplot(2, 1, 1)
    plt.boxplot(data, labels=model_name_list, showfliers=True)
    plt.xticks([])
    plt.title('Reconstruction with outliers')
    plt.ylabel('distance btw model and ref')

    plt.subplot(2, 1, 2)
    plt.boxplot(data, labels=model_name_list, showfliers=False, vert=True)
    plt.xticks(rotation=15)
    plt.title('Reconstruction with no outliers')
    plt.ylabel('distance btw model and ref')
    plt.suptitle('Boxplot for ' + str(model_name[0]))
    plt.show()

    descr_stat = data.describe()
    print(descr_stat)

    mean_test = pd.DataFrame(descr_stat.loc['mean', :])
    mean_test = abs(mean_test)
    print('Minimum abs mean is : ')
    mini = min(abs(mean_test.loc[:, 'mean']))
    print(round(mini, 6))
    print('it is from : ')
    temp = mean_test[mean_test == mini].stack().index.tolist()
    min_mean = temp[0][0]
    print(min_mean)

    # # Check anova's asumption
    # # 1. Normality -> normality accepted if pvalue < 0.05
    # for name, values in data.iteritems():
    #     print(name)
    #     print(stats.shapiro(values))
    #
    # # 2. equal variance
    list_combinations = list()
    for n in range(2, len(model_name_list) + 1):
        list_combinations += list(combinations(model_name_list, n))

    barlett_df = pd.DataFrame(list_combinations, columns=['col1', 'col2', 'col3', 'col4'])
    barlett_pv_list = []
    barlett_int = []
    for index, row in barlett_df.iterrows():
        if pd.isnull(row['col3']) and pd.isnull(row['col4']):
            barlett = stats.bartlett(data[str(row['col1'])], data[str(row['col2'])])
            barlett_pv_list.append(barlett.pvalue)
        elif pd.isnull(row['col4']):
            barlett = stats.bartlett(data[str(row['col1'])], data[str(row['col2'])], data[str(row['col3'])])
            barlett_pv_list.append(barlett.pvalue)
        else:
            barlett = stats.bartlett(data[str(row['col1'])], data[str(row['col2'])], data[str(row['col3'])],
                                     data[str(row['col4'])])
            barlett_pv_list.append(barlett.pvalue)
        if barlett.pvalue < 0.05:
            barlett_int.append('RHO')
        elif barlett.pvalue > 0.05:
            barlett_int.append('AHO')

    barlett_df['pvalue'] = barlett_pv_list
    barlett_df['rejectHO?'] = barlett_int
    print(barlett_df)

    if 'AHO' in barlett_df.iloc[-1, -1]:
        print('OK ANOVA + tukey post hoc')
        # Anova : if pvalue < 0.05; RH0 :  at least one of the means of the groups is different from the others.
        anova = stats.f_oneway(data['C2M_pc_Circle'], data['C2M_pc_NADIR_pitch-45'], data['C2M_pc_ov50'],
                               data['C2M_pc_Sphere'])
        print(anova)
        print("si pv < 0.05, RHO : les différentes reconstruction n'ont pas la même moyenne")

        # perform Tukey's test
        tukey = pairwise_tukeyhsd(endog=tukeys['C2M_signed_distances'], groups=tukeys['name'], alpha=0.05)

        tukey_df = pd.DataFrame(data=tukey._results_table.data[1:], columns=tukey._results_table.data[0])

        print(tukey)
        print(
            "si pv < 0.05, RHO : reject = TRUE : les différentes de moyenne entre group 1 et group 2 sont significative")

        # tukey.plot_simultaneous(comparison_name=min_mean, xlabel='difference btw mean of sphere and current mean')
        # rows = tukey.summary().data[1:]
        # print(rows)
        # plt.hlines(range(len(rows)), [row[4] for row in rows], [row[5] for row in rows])
        # plt.vlines(0, -1, len(rows) - 1, linestyles='dashed')
        # plt.gca().set_yticks(range(len(rows)))
        # plt.gca().set_yticklabels([f'{x[0]}-{x[1]}' for x in rows])
        # plt.show()

        for index, row in tukey_df.iterrows():
            if (row['group1'] == min_mean or row['group2'] == min_mean) and row['reject']:
                print(row['meandiff'])
                res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.01, upp=0.01,
                                    usevar='unequal')
                overall_p = res[0]
                if overall_p < 0.05:
                    print(str(row['group1']) + " and " + str(
                        row['group2']) + " are Equivalent in [-0.01;0.01] interval")
                elif overall_p > 0.05:
                    res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.05, upp=0.05,
                                        usevar='unequal')
                    overall_p = res[0]
                    if overall_p < 0.05:
                        print(
                            str(row['group1']) + " and " + str(
                                row['group2']) + " are Equivalent in [-0.05;0.05] interval")
                    elif overall_p > 0.05:
                        res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.1,
                                            upp=0.1,
                                            usevar='unequal')
                        overall_p = res[0]
                        if overall_p < 0.05:
                            print(
                                str(row['group1']) + " and " + str(
                                    row['group2']) + " are Equivalent in [-0.1;0.1] interval")
                        elif overall_p > 0.05:
                            print(
                                str(row['group1']) + " and " + str(
                                    row['group2']) + " are not Equivalent in [-0.1;0.1] interval")
                print('------------')
            elif (row['group1'] == min_mean or row['group2'] == min_mean) and not row['reject']:
                wttest = stats.ttest_ind(data[str(row['group1'])], data[str(row['group2'])], equal_var=False)
                if wttest.pvalue < 0.05:
                    print(str(row['group1']) + " and " + str(row['group2']) + " have the same mean ! ")
                elif wttest.pvalue > 0.05:
                    print(str(row['group1']) + " and " + str(row['group2']) + " does NOT have the same mean ! ")
                    print('Error from tukey test due to small sample')
                print('------------')
    elif 'RHO' in barlett_df.iloc[-1,-1]:
        print("si pv < 0.05, RHO : les différents reconstruction n'ont pas la mm variance")

        '''In general, a one-way ANOVA is considered to be fairly robust against violations
        of the equal variances assumption as long as each group has the same sample size.
        rule of thumb that the ANOVA is robust to heterogeneity of variance so long as
        the largest variance is not more than 4 times the smallest variance.
        '''
        var_test = pd.DataFrame(descr_stat.loc['std', :])
        var_test = var_test ** 2
        mini = min(var_test['std'])
        maxi = max(var_test['std'])
        print('Minimum var is : ' + str(round(mini, 6)) + '  and max var is : ' +str(round(maxi, 6)))
        if 4 * mini >= maxi:
            print('OK ANOVA + tukey post hoc')
            # Anova : if pvalue < 0.05; RH0 :  at least one of the means of the groups is different from the others.
            anova = stats.f_oneway(data['C2M_pc_Circle'], data['C2M_pc_NADIR_pitch-45'], data['C2M_pc_ov50'],
                                   data['C2M_pc_Sphere'])
            print(anova)
            print("si pv < 0.05, RHO : les différentes reconstruction n'ont pas la même moyenne")

            # perform Tukey's test
            tukey = pairwise_tukeyhsd(endog=tukeys['C2M_signed_distances'], groups=tukeys['name'], alpha=0.05)

            tukey_df = pd.DataFrame(data=tukey._results_table.data[1:], columns=tukey._results_table.data[0])

            print(tukey)
            print(
                "si pv < 0.05, RHO : reject = TRUE : les différentes de moyenne entre group 1 et group 2 sont significative")

            # tukey.plot_simultaneous(comparison_name=min_mean, xlabel='difference btw mean of sphere and current mean')
            # rows = tukey.summary().data[1:]
            # print(rows)
            # plt.hlines(range(len(rows)), [row[4] for row in rows], [row[5] for row in rows])
            # plt.vlines(0, -1, len(rows) - 1, linestyles='dashed')
            # plt.gca().set_yticks(range(len(rows)))
            # plt.gca().set_yticklabels([f'{x[0]}-{x[1]}' for x in rows])
            # plt.show()

            for index, row in tukey_df.iterrows():
                if (row['group1'] == min_mean or row['group2'] == min_mean) and row['reject']:
                    print(row['meandiff'])
                    res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.01, upp=0.01,
                                        usevar='unequal')
                    overall_p = res[0]
                    if overall_p < 0.05:
                        print(str(row['group1']) + " and " + str(
                            row['group2']) + " are Equivalent in [-0.01;0.01] interval")
                    elif overall_p > 0.05:
                        res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.05,
                                            upp=0.05,
                                            usevar='unequal')
                        overall_p = res[0]
                        if overall_p < 0.05:
                            print(
                                str(row['group1']) + " and " + str(
                                    row['group2']) + " are Equivalent in [-0.05;0.05] interval")
                        elif overall_p > 0.05:
                            res = ssw.ttost_ind(data[str(row['group1'])], data[str(row['group2'])], low=-0.1,
                                                upp=0.1,
                                                usevar='unequal')
                            overall_p = res[0]
                            if overall_p < 0.05:
                                print(
                                    str(row['group1']) + " and " + str(
                                        row['group2']) + " are Equivalent in [-0.1;0.1] interval")
                            elif overall_p > 0.05:
                                print(
                                    str(row['group1']) + " and " + str(
                                        row['group2']) + " are not Equivalent in [-0.1;0.1] interval")
                    print('------------')
                elif (row['group1'] == min_mean or row['group2'] == min_mean) and not row['reject']:
                    wttest = stats.ttest_ind(data[str(row['group1'])], data[str(row['group2'])],
                                             equal_var=False)
                    if wttest.pvalue < 0.05:
                        print(str(row['group1']) + " and " + str(row['group2']) + " have the same mean ! ")
                    elif wttest.pvalue > 0.05:
                        print(str(row['group1']) + " and " + str(
                            row['group2']) + " does NOT have the same mean ! ")
                        print('Error from tukey test due to small sample')
                    print('------------')

        else:
            print('PAS OK ANOVA, go Welch ANOVA + games-howell posthoc')

            welch_anova = pg.welch_anova(dv='C2M_signed_distances', between='name', data=tukeys)
            print(welch_anova)
            gh_posthoc = pg.pairwise_gameshowell(dv='C2M_signed_distances', between='name', data=tukeys)
            print(gh_posthoc)
            for index, row in gh_posthoc.iterrows():
                if (row['A'] == min_mean or row['B'] == min_mean) and row['pval'] < 0.05:
                    print(row['diff'])
                    res = ssw.ttost_ind(data[str(row['A'])], data[str(row['B'])], low=-0.01, upp=0.01,
                                        usevar='unequal')
                    overall_p = res[0]
                    if overall_p < 0.05:
                        print(str(row['A']) + " and " + str(
                            row['B']) + " are Equivalent in [-0.01;0.01] interval")
                    elif overall_p > 0.05:
                        res = ssw.ttost_ind(data[str(row['A'])], data[str(row['B'])], low=-0.05, upp=0.05,
                                            usevar='unequal')
                        overall_p = res[0]
                        if overall_p < 0.05:
                            print(
                                str(row['A']) + " and " + str(
                                    row['B']) + " are Equivalent in [-0.05;0.05] interval")
                        elif overall_p > 0.05:
                            res = ssw.ttost_ind(data[str(row['A'])], data[str(row['B'])], low=-0.1, upp=0.1,
                                                usevar='unequal')
                            overall_p = res[0]
                            if overall_p < 0.05:
                                print(
                                    str(row['A']) + " and " + str(
                                        row['B']) + " are Equivalent in [-0.1;0.1] interval")
                            elif overall_p > 0.05:
                                print(
                                    str(row['A']) + " and " + str(
                                        row['B']) + " are not Equivalent in [-0.1;0.1] interval")
                    print('------------')
                elif (row['A'] == min_mean or row['B'] == min_mean) and row['pval'] < 0.05:
                    wttest = stats.ttest_ind(data[str(row['A'])], data[str(row['B'])], equal_var=False)
                    if wttest.pvalue < 0.05:
                        print(str(row['A']) + " and " + str(row['B']) + " have the same mean ! ")
                    elif wttest.pvalue > 0.05:
                        print(str(row['A']) + " and " + str(row['B']) + " does NOT have the same mean ! ")
                        print('Error from tukey test due to small sample')
                    print('------------')

# '''
# La moyenne des dist entre la reconstruction et le modèle de ref se situe dans l'intervalle +-1cm
# Si on fait le cercle à la place de la sphère, on perd max 1 cm de justesse géométrique, mesurée par sa moyenne
#'''