import pyCloudCompare as cc
import os
import pandas as pd
import scipy.stats as stats
import matplotlib.pyplot as plt
import seaborn as sns
import glob

sns.set_theme(style="ticks", color_codes=True)
model_path_in = r"C:\Users\Thomas\Documents\Thomas_script_test2\model_test"
project_path = r"C:\Users\Thomas\Documents\Thomas_script_test2"
cli = cc.CloudCompareCLI()

os.chdir(model_path_in)
models = glob.glob('*.obj')
percent_list = []
for model in models:
    name = model.split('.')
    print(name[0], '---------------------------')
    model_path = os.path.join(project_path, name[0])
    mesh = os.path.join(model_path_in, model)
    for root, dirs, files in os.walk(model_path):
        for name in files:
            if name.startswith("reconstruction_pc_") and name.endswith(".las") and not name.endswith("CRS_WGS84.las"):
                rec = os.path.join(root, name)
                folder_name = os.path.basename(os.path.dirname(rec))

                cmd = cli.new_command()
                cmd.silent()  # Disable console
                cmd.open(os.path.join(model_path_in, model)) # .obj model
                mesh = cmd.sample_mesh(method='points', parameter=1000000)
                rec = cmd.open(os.path.join(root, name)) # .las reconstruction
                cmd.match_centers()
                cmd.icp(overlap=80, adjust_scale=True, reference_is_first=True)

                cmd.c2c_dist()
                cmd.cloud_export_format(cc.CLOUD_EXPORT_FORMAT.ASCII, extension="xyz", add_header=True)
                cmd.save_clouds(os.path.join(root,"completude_"+ str(folder_name)+ ".xyz"), os.path.join(root,"2ndCloud.xyz"))
                print(cmd)
                cmd.execute()
                os.remove(os.path.join(root,"2ndCloud.xyz"))
                print('----------------')

                df = pd.read_csv(os.path.join(root,"completude_"+ str(folder_name)+ ".xyz"), header=0, delimiter='\s+', index_col=False)
                df = df.reset_index()
                completude = df.loc[:, 'C2C_absolute_distances']
                print(len(completude))
                completude_t_20cm = completude[completude < 0.2]

                ax = sns.histplot(data=completude_t_20cm, kde=False, bins=50, stat='density').set(title=str(folder_name)+' reconstruction')
                #ax.lines[0].set_color('crimson')
                plt.show()

                completude_percent = (len(completude_t_20cm) / len(completude)) * 100
                print(completude_percent)
                print('----------------')
                percent_list.append([folder_name, completude_percent])

    # remove .bin and .txt files in model_path
    for filename in os.listdir(model_path_in):
        if filename.endswith(".bin"):
            # Add your "dirname" to the file path
            os.remove(os.path.join(model_path_in, filename))
        if filename.endswith(".txt"):
            # Add your "dirname" to the file path
            os.remove(os.path.join(model_path_in, filename))

    df = pd.DataFrame(percent_list)
    df.columns = ['reconstruction','completude_percentage']
    print(df)
    df.to_csv((os.path.join(model_path, 'completude_percent_' + str(name[0]) + '.csv')))

    sns.catplot(x='reconstruction', y='completude_percentage', kind ='bar', data=df, palette="pastel")\
        .set(title='% completude by reconstruction for ' + str(name[0]))
    plt.show()