# Import librairies ----------------------------------------------------------------------------------------------------
import pandas as pd
import glob
import os
from pathlib import Path
import matplotlib.pyplot as plt
import seaborn as sns

# get license
license_path = r"C:\Program Files\Agisoft\Metashape Pro\metashape.lic"
os.environ['agisoft_LICENSE'] = str(Path(license_path))
print('License found : ', os.environ['agisoft_LICENSE'])
import Metashape

model_path_in = r"E:\thomas_TFE3\final_church"
project_path = r"E:\thomas_TFE3\Thomas_script_testSave"
doc = Metashape.Document()

os.chdir(model_path_in)
models = glob.glob('*.obj')
for model in models:
    name = model.split('.')
    print(name[0], '---------------------------')
    model_path = os.path.join(project_path, name[0])
    doc.open(path=os.path.join(model_path, "reconstructed_model_" + str(name[0] + ".psx")))

    certitude_percent = []
    for chunk in doc.chunks:
        if chunk.dense_cloud:
            print(chunk)
            count0 = chunk.dense_cloud.point_count
            chunk.dense_cloud.setConfidenceFilter(0, 2)  # remove point that have confidence below 2
            chunk.dense_cloud.removePoints(list(range(128)))  # removes all "visible" points of the dense cloud
            chunk.dense_cloud.resetFilters()
            chunk.dense_cloud.selectPointsByColor(color=[255, 255, 255], tolerance=99, channels='RGB')
            chunk.dense_cloud.removeSelectedPoints()
            count = chunk.dense_cloud.point_count
            percent_cert = (count / count0) * 100
            print(str(percent_cert), ' %')
            certitude_percent.append([str(chunk.label), percent_cert])

    df_confidence = pd.DataFrame(certitude_percent)
    df_confidence.columns = ['chunk_name', 'percent']
    df_confidence.to_csv((os.path.join(model_path, 'confidence_percent_' + str(name[0]) + '.csv')))

    sns.catplot(x='chunk_name', y='percent', kind='bar', data=df_confidence, palette="pastel") \
        .set(title='% confidence by reconstruction for ' + str(name[0]))
    plt.show()


